#!/usr/bin/env python

import sys
import numpy as np
import cv2
import ut

wh = lambda img: ( lambda (h, w, d): (w, h) )( img.shape )
resize = lambda img, w, h: img if wh(img) == (w, h) else cv2.resize( img, (w, h) )

def new_imgs():
	d = {}
	e = ut.Empty()

	def get(fn):
		if fn not in d:
			d[fn] = cv2.imread(fn)
		return d.get(fn)
	e.get = get

	e.wh = lambda fn: wh( get(fn) )
	e.in_img = lambda fn, x, y: ( lambda (w, h): 0 <= x and x < w and 0 <= y and y < h )( e.wh(fn) )
	e.col = lambda fn, x, y, def_col=[128,128,128]: get(fn)[y, x] if e.in_img(fn, x, y) else def_col

	return e

imgs = new_imgs()

def new_wimg(w, h, def_col=[0,0,0]):
	e = ut.Empty()
	e.img = np.empty( (h, w, 3), dtype=np.uint8 )

	def clear():
		e.img[:] = def_col
	e.clear = clear
	clear()

	e.in_img = lambda x, y: 0 <= x and x < w and 0 <= y and y < h

	def set(x, y, col):
		if e.in_img(x, y):
			e.img[y, x] = col
	e.set = set

	e.col = lambda x, y: e.img[y, x] if e.in_img(x, y) else def_col
	e.write = lambda fn: cv2.imwrite(fn, e.img)

	def show(title='', sec=-1, zm=1):
		img = e.img
		if zm != 1:
			img = resize( img, int(w*zm), int(h*zm) )
		cv2.imshow(title, img)
		msec = 0 if sec < 0 else max( 1, int(sec * 1000) )
		cv2.waitKey(msec)
	e.show = show

	def line(sx, sy, ex, ey, col):
		r_d = lambda s, e: ( e < s, abs(e - s) )
		(rx, dx) = r_d(sx, ex)
		(ry, dy) = r_d(sy, ey)
		rxy = dx < dy
		(a, b) = (dy, dx) if rxy else (dx, dy)
		sum = j = 0;
		for i in range(a):
			sum += b;
			if sum > a:
				sum -= a
				j += 1
			(x, y) = (j, i) if rxy else (i, j)
			x = sx + (-x if rx else x)
			y = sy + (-y if ry else y)
			set(x, y, col)
	e.line = line

	return e

def new_video(codec, fn, fps, w, h, zm=1.0, def_col=[0,0,0]):
	e = new_wimg(w, h, def_col)
	f = cv2.VideoWriter_fourcc if hasattr(cv2, 'VideoWriter_fourcc') else cv2.cv.CV_FOURCC
	fourcc = f(*codec)
	(vw, vh) = ( int(w*zm), int(h*zm) )
	writer = cv2.VideoWriter( fn, fourcc, fps, (vw, vh) )
	e.add_img = lambda : writer.write( resize( e.img, vw, vh ) )
	e.show_zm = lambda title='', sec=-1: e.show(title, sec, zm)
	e.fin = lambda : writer.release()
	return e

def imgs_to_video(img_name, video_name, fps, zm):
	cmd = 'ls {}[0-9]*.jpg'.format(img_name)
	lst = ut.cmd_exec(cmd).strip().split('\n')

	imgs = new_imgs()
	(w, h) = imgs.wh(lst[0])
	codec = 'H264'
	fn = video_name + '.mp4'
	video = new_video(codec, fn, fps, w, h, zm)

	for fn in lst:
		video.img[:] = resize( imgs.get(fn), w, h )[:]
		video.add_img()
	video.fin()

if __name__ == "__main__":
	if len(sys.argv[1:]) < 2:
		print 'Usage: {} img_name video_name fps=<fps> zm=<zm> codec=<codec>'.format(sys.argv[0])
		print '  fps is int value (default 30)'
		print '  zm is float value (default 1.0)'
		sys.exit(0)

	img_name = sys.argv[1]
	video_name = sys.argv[2]
	fps = ut.arg_i('fps', 30)
	zm = ut.arg_f('zm', 1.0)
	imgs_to_video(img_name, video_name, fps, zm)
# EOF
