#!/usr/bin/env python

import math

linear_conv = lambda s, sa, sb, da, db: da + (s - sa) * (db - da) / float(sb - sa) if sa != sb else (da + db) * 0.5

deg_to_rad = lambda deg: deg * math.pi / 180
rad_to_deg = lambda rad: rad * 180 / math.pi

def v3_det(v3):
	(x, y, z) = v3
	(xx, xy, xz) = x
	(yx, yy, yz) = y
	(zx, zy, zz) = z
	return xx*yy*zz + xy*yz*zx + xz*yx*zy - xx*yz*zy - xz*yy*zx - xy*yx*zz

def v3_rev(v3):
	d = v3_det(v3)
	if d == 0:
		return []
	d_ = 1.0 / d

	(x, y, z) = v3
	(xx, xy, xz) = x
	(yx, yy, yz) = y
	(zx, zy, zz) = z

	tx = [ yy*zz - zy*yz, zy*xz - xy*zz, xy*yz - yy*xz ]
	ty = [ zx*yz - yx*zz, xx*zz - zx*xz, yx*xz - xx*yz ]
	tz = [ yx*zy - zx*yy, zx*xy - xx*zy, xx*yy - yx*xy ]

	return map( lambda v: map( lambda e: e * d_, v ), [tx, ty, tz] )

###

def quadratic_formula(a, b, c):
	d = b * b - 4 * a * c
	if d < 0:
		return []
	a2 = 2.0 * a
	if d == 0:
		return [ -b / a2 ]
	d_ = math.sqrt(d)
	return [ (-b + d_) / a2, (-b - d_) / a2 ]

###

def interp3_abcd(v0, v1, k0=None, k1=None):
	k = v1 - v0;
	k0_ = k0
	k1_ = k1
	if k0 == None:
		k0 = k
	if k1 == None:
		k1 = k
	if k0_ == None:
		k0 = 2*k - k1
	if k1_ == None:
		k1 = 2*k - k0
	d = v0
	c = k0
	b = 3*(v1 - d) - k1 - 2*c
	a = v1 - b - c - d
	return (a, b, c, d)

def interp3(v0, v1, k0, k1, t_in):

#	v = at^3 + bt^2 + ct + d
#	k = 3at^2 + 2bt + c
#	  t : 0 -> 1
#
#	v0 = d
#	k0 = c
#	v1 = a + b + c + d
#	k1 = 3a + 2b + c
#	
#	3v1 = 3a + 3b + 3c + 3d
#	k1  = 3a + 2b + c	
#
#	3v1 - k1 = b + 2c + 3d
#	b = 3v1 - k1 - 2c - 3d
#          = 3(v1 - d) - k1 - 2c
#
#	( b = 3(v1 - v0) - k1 - 2k0 )
#
#	v1 = a + b + c + d
#	a = v1 - b - c - d

	(a, b, c, d) = interp3_abcd(v0, v1, k0, k1)
	v = ((a * t_in + b) * t_in + c ) * t_in + d
	k = (3 * a * t_in + 2 * b) * t_in + c
	return (v, k)

# EOF
