#!/usr/bin/env python

import sys
import os
import time
import subprocess

arg_s = lambda k, def_val='': next( ( s[ len(k)+1: ] for s in sys.argv if s.startswith(k+'=') ), def_val )
arg_v = lambda fs, k, def_val: ( lambda s: fs(s) if s else def_val )( arg_s(k) )
arg_i = lambda k, def_val: arg_v( int, k, def_val )
arg_f = lambda k, def_val: arg_v( float, k, def_val )

def dic_set(d, k, v):
	d[k] = v
	return v

dic_cache = lambda d, k, f_v: d.get(k) if k in d else dic_set( d, k, f_v() )

if_not_set = lambda d, k, v: dic_cache(d, k, lambda : v)

###

def cls_to_dic(c):
	ks = filter( lambda k: not k.startswith('_'), dir(c) )
	return dict( map( lambda k: ( k, getattr(c, k) ), ks ) )

def dic_to_cls(d, c=None):
	if not c:
		c = Empty()
	map( lambda (k, v): setattr(c, k, v), d.items() )
	return c

extends = lambda pcls, c=None: dic_to_cls( cls_to_dic(pcls), c )

class Empty:
	def __init__(self, pcls=None):
		if pcls:
			dic_to_cls( cls_to_dic(pcls), self )

get_typ = lambda targ, typ='p': targ.typ if isinstance(targ, Empty) else typ

###

now_sec = time.time
sleep = time.sleep

#str_dt = lambda sec: time.asctime( time.localtime(sec) )
def str_dt(sec):
	t = time.localtime(sec)
	lst = [ t.tm_year, t.tm_mon, t.tm_mday, t.tm_hour, t.tm_min, t.tm_sec ]
	return '{}/{:02}/{:02} {:02}:{:02}:{:02}'.format(*lst)

def str_hms(sec):
	si = int(sec)
	m = si / 60
	s = si % 60 + int( (sec - si) * 100 ) * 0.01
	h = m / 60
	m %= 60
	lst = [ (s, 's') ]
	if m:
		lst.insert( 0, (m, 'm') )
	if h:
		lst.insert( 0, (h, 'h') )
	return ' '.join( map( lambda (v, s): str(v)+s, lst ) )

###

def new_cnt(n, id_str=''):
	if id_str:
		id_str += ' : '

	e = Empty()
	e.n = n
	e.i = 0
	e.st_sec = None
	e.total_sec = None
	e.show_sec = None
	e.show_interval = 1.0

	def up():
		now = now_sec()
		if e.st_sec == None:
			e.st_sec = now
		else:
			e.total_sec = now - e.st_sec
			e.i += 1
		run = e.i < e.n
		if e.total_sec != None and run:
			e.total_sec = e.total_sec * n / e.i
		show_stat()
		return run
	e.up = up

	e.cur = lambda : e.i

	def fin_str():
		if e.total_sec == None:
			return 'not start yet'
		if e.i >= n:
			return 'fin ' + str_hms(e.total_sec)
		rest = e.total_sec - ( now_sec() - e.st_sec )
		return 'rest {} : {}'.format( str_hms(rest), str_dt( e.st_sec + e.total_sec ) )

	stat_str = lambda : '{}{}/{}({}%) : {}'.format( id_str, e.i, n, 1000*e.i/n*0.1, fin_str() )

	def show_stat():
		now = now_sec()
		if e.show_sec == None or e.i >= n or now - e.show_sec >= e.show_interval:
			e.show_sec = now
			print stat_str()

	return e

def new_cnt_wait(id_str=''):
	sec = arg_f('sec', 1.0)
	fps = arg_f('fps', 30.0)
	n = arg_i( 'n', int( fps * sec ) )

	cnt = new_cnt(n, id_str)
	e = Empty(cnt)

	e.sec = sec
	e.fps = fps

	e.cur_sec = lambda : sec * cnt.cur() / n

	e.wait_sec = lambda : e.cur_sec() - ( now_sec() - cnt.st_sec )
	def wait():
		w = e.wait_sec()
		if w > 0:
			sleep(w)
	e.wait = wait

	return e

def new_cnt_wh(w, h, id_str=''):
	n = w * h
	cnt = new_cnt(n, id_str)
	e = Empty(cnt)
	e.cur = lambda : ( lambda i: ( i%w, i/w ) )( cnt.cur() )
	return e

###

def cmd_exec(cmd):
	try:
		return subprocess.check_output(cmd, shell=True)
	except:
		return ''
###

exists = lambda path: os.path.exists(path)

def mkdir_if_not(file_path):
	(d, n) = os.path.split(file_path)
	if not exists(d):
		cmd_exec( "mkdir -p '{}'".format(d) )

# EOF
