#!/usr/bin/env python

import sys
import ut
import mt
import v
import line

#	[ (v, t), (v, t), ... (v, t) ]
#	or
#	[ (v, t), (v, t), ... (v, None) ]
#
#	v: value
#	t: sec interval for next v

def get_idx_t(ts, t):
	s = sum(ts)
	t -= int( float(t) / s ) * s
	s = 0
	for i in range( len(ts) ):
		if s <= t and t <= s + ts[i]:
			break
		s += ts[i]
	t -= s
	return (i, t)

def get_k(lst, i):
	(v, t) = lst[i]
	if t == None:
		return None
	j = (i + 1) % len(lst)
	(v2, _) = lst[j]
	return float(v2 - v) / t

def get(lst, t, itp3=False):
	# itp3 is using mt.interp3

	(vs, ts) = zip(*lst)
	ts_ = ts if ts[-1] != None else ts[:-1]
	(idx, t) = get_idx_t(ts_, t)
	n = len(lst)
	if itp3:
		ka = get_k( lst, (idx-1+n)%n )
		kb = get_k( lst, idx )
		kc = get_k( lst, (idx+1)%n )
				
		f = lambda k, k_, ti: None if k == None or k_ == None else (k + k_) * 0.5 * ti
		ti = ts[idx]
		k0 = f(ka, kb, ti)
		k1 = f(kb, kc, ti)

		(v, k) = mt.interp3( vs[idx], vs[(idx+1)%n], k0, k1, float(t)/ti )
		return v

	return mt.linear_conv( t, 0, ts[idx], vs[idx], vs[(idx+1)%n] )

def liss(ps, pe, t3):
	lsts = map( lambda (s, e, t): [ (s, t*0.5), (e, t*0.5) ], zip(ps, pe, t3) )
	return lambda t, itp3=True: map( lambda lst: get(lst, t, itp3), lsts )
	
def liss_eye():
# sys.argv
# 'eyep=[0,-10,0],20,40' 'eyet=[0,0,0],1,50' init_sec=5
# [x,y,z],r,t
#
# [x,y,z]: default [0,0,0]
# r: default 0
# t: default 1.0 (sec)

	def p_r_t(key):
		lst = [ [0,0,0],0,1 ]
		lst_ = eval( '[' + ut.arg_s(key) + ']' )
		(n, n_) = map( len, (lst, lst_) )
		return map( lambda i: lst_[i] if i < n_ else lst[i], range(n) )
	
	def liss_get(key):
		(p, r, t) = p_r_t(key)
		ps = v.op1('-', p, r)
		pe = v.op1('+', p, r)
		t3 = (t, t*4/3, t*5/3)
		return liss(ps, pe, t3)

	f_p = liss_get('eyep')
	f_t = liss_get('eyet')
	init_sec = ut.arg_f('init_sec', 0.0)
	return lambda t, itp3=True: line.new_p2( f_p(t+init_sec, itp3), f_t(t+init_sec, itp3) )

def test(lst, itp3=False, end_t=4, stp_t=0.25):
	print lst, 'itp3=', itp3
	t = 0
	while t <= end_t:
		v = get(lst, t, itp3)
		print (t, v)
		t += stp_t
	s = 0
	for (v, t) in lst:
		if t != None:
			v_ = get(lst, s, itp3)
			if v_ != v:
				print 'NG t={} v={} (!={})'.format(t, v_, v)
				sys.exit(-1)
			s += t

if __name__ == "__main__":
	test( [(0,1),(1,1)] )
	test( [(0,1),(1,1)], True )
	test( [(0,1),(1,None)] )
	test( [(0,1),(1,None)], True )

	test( [(0,1),(2,3)] )
	test( [(0,1),(2,3)], True )
	test( [(0,1),(2,None)] )
	test( [(0,1),(2,None)], True )

	test( [(3,2),(1,1)] )
	test( [(3,2),(1,1)], True )
	test( [(3,2),(1,None)] )
	test( [(3,2),(1,None)], True )

	print 'OK'
# EOF
