#!/usr/bin/env python

import v
import line
import ax
import lstx

def wf_ps_idxs(d):
	ps_to_idxs = lambda ps: map( lambda i: ( i, (i+1)%n ), range( len(ps) ) )
	rv_ps = lambda ps: ( ps, ps_to_idxs(ps) )

	kind = d.get('kind')

	if kind == 'square':
		n = 4
		ps = map( lambda i: ax.rot_z(i*90).tr('l2g', [1,1,0]), range(n) )
		return rv_ps(ps)
	if kind == 'triangle':
		n = 3
		ps = [ [0,1,0],[-1,0,0],[1,0,0] ]
		return rv_ps(ps)
	if kind == 'poly_n':
		n = d.get('n', 3)
		deg = 360.0 / n
		ps = map( lambda i: ax.rot_z( (i+0.5)*deg ).tr('l2g', [0,-1,0]), range(n) )
		return rv_ps(ps)
	if kind == 'circle':
		d = { 'kind': 'poly_n', 'n': 16 }
		return wf_ps_idxs(d)
	if kind == 'ball':
		d = { 'kind': 'circle' }
		c3 = map( lambda i: wf_ps_idxs(d), range(3) )

		(ps, idxs) = c3[1]
		n = len(ps)
		ps = map( lambda p: ax.rot_x(90).tr('l2g', p), ps )
		idxs = map( lambda idx: map( lambda id: id + n, idx ), idxs )
		c3[1] = (ps, idxs)

		(ps, idxs) = c3[2]
		n *= 2
		ps = map( lambda p: ax.rot_y(90).tr('l2g', p), ps )
		idxs = map( lambda idx: map( lambda id: id + n, idx ), idxs )
		c3[2] = (ps, idxs)

		return map( lambda xs_lst: sum( xs_lst, [] ), zip(*c3) )

	return ([],[])

def nv_clip(nv, pa, pb):
	nv_line = line.new( [0,0,0], nv )
	(in_a, in_b) = map( lambda p: nv_line.is_plane_side(p), [pa, pb] )
	if in_a and in_b:
		return [pa, pb]
	if not in_a and not in_b:
		return []
	pc = nv_line.cross_plane_line( line.new_p2(pa, pb) )
	return [pa, pc] if in_a else [pc, pb]

def eye_clip(eye2g, wh2eye, sc_sz, pa, pb):
	(w, h, d) = sc_sz
	n = 4
	p4 = [ [0,0,-d], [w,0,-d], [w,h,-d], [0,h,-d] ]
	p4 = map( lambda p: lstx.tr(wh2eye, 'l2g', p), p4 )
	for i in range(n):
		nv = v.p3_to_nv( [0,0,0], p4[i], p4[(i+1)%n] )
		r = nv_clip(nv, pa, pb)
		if not r:
			return []
		(pa, pb) = r
	return [pa, pb]

def draw(data, eye2g, wh2eye, sc_sz, video):
	col = v.all(255) # white
	for d in data:
		ps = []
		idxs = []
		(ps, idxs) = wf_ps_idxs(d)

		ps = map( lambda p: lstx.tr( d.get('l2g'), 'l2g', p ), ps )
		ps = map( lambda p: eye2g.tr('g2l', p), ps )

		p2s = map( lambda (i, j): eye_clip( eye2g, wh2eye, sc_sz, ps[i], ps[j] ), idxs )
		p2s = filter( lambda p2: p2, p2s )
		p2s = map( lambda p2: map( lambda p: lstx.tr( wh2eye, 'g2l', p), p2 ), p2s )
		for (pa, pb) in p2s:
			(ax, ay, _) = pa
			(bx, by, _) = pb
			video.line( int(ax), int(ay), int(bx), int(by), col )
# EOF
